from .core import ClusterBase, PROD_API_URL
from .schema import ClusterModel


class Cluster():
    def __init__(self, cluster={}, client=None, url=PROD_API_URL):
        self.cluster = ClusterModel(**cluster)
        self.class_ = self.cluster.class_
        self.description = self.cluster.description
        self.client = client or ClusterBase(url=url)

    def add_machine(self, machine, validate=True, cluster=None):
        """
        Add a machine to a cluster
        """

        cluster = cluster or self.cluster.cluster or None

        return self.client._post(path=f'/cluster/{cluster}/add/{machine}', params={'validate': validate})[0]

    def remove_machine(self, machine, validate=True, cluster=None):
        """
        Remove a machine from a cluster
        """

        cluster = cluster or self.cluster.cluster or None

        return self.client._delete(path=f'/cluster/{cluster}/remove/{machine}', params={'validate': validate})

    def list_all_clusters(self):
        """
        Returns an array of all clusters
        """

        return [ClusterModel(**{'cluster': cluster}) for cluster in self.client._get(path='/cluster/all')]

    def get(self, cluster=None, params=False):
        """
        Fetch information about this cluster. Optionally fetch cluster params.
        """

        cluster = cluster or self.cluster.cluster

        result = ClusterModel(
            **self.client._get(path=f'/cluster/{cluster}', params={'params': params}))
        self.cluster = result
        self.class_ = self.cluster.class_
        self.description = self.cluster.description

        return result

    def delete(self, cluster=None, cleanup=False):
        """
        Delete a cluster. "cleanup" defines whether the cluster parameters
        should also be deleted, or if that will happen out-of-band
        """

        cluster = cluster or self.cluster.cluster

        return self.client._delete(path=f'/cluster/{cluster}', params={'cleanup': cleanup})

    def create(self, cluster=None, class_=None, description=None):
        """
        Create or update a cluster. Return status code.
            200 = updated/already existed
            201 = created
        """

        new_cluster = {
            'cluster': cluster or self.cluster.cluster,
            'class': class_ or self.cluster.class_,
            'description': description or self.cluster.description}

        result = self.client._post(path=f'/cluster', data=new_cluster)
        return ClusterModel(**result[0]), result[1]

    def machines(self, cluster=None, location=None, all_machines=True):
        """
        Returns an array of cluster machines. all_machines toggles whether to include machines marked as "down"
        """

        cluster = cluster or self.cluster.cluster

        params = {'all_machines': all_machines}

        if location:
            params['location'] = location

        try:
            return self.client._get(path=f'/cluster/machines/{cluster}', params=params)
        except Exception as err:
            return []
