from typing import List, Union, Optional
from datetime import datetime, date
from ipaddress import IPv4Address
from enum import Enum
from pydantic import BaseModel

# TODO: find some way to track upstream API models


class MachineModel(BaseModel):
    class Config:
        use_enum_values = True

    machine: str = None
    location: str = None
    domain: str = None
    ip: Union[IPv4Address, str] = None
    type: str = None

    class statusenum(Enum):
        up = 'up'
        down = 'down'

    status: statusenum = None
    comment: str = None
    hardware_description: str = None

    class realmenum(Enum):
        logical = 'logical'
        physical = 'physical'

    realm: realmenum = None
    masq: str = None
    rack: str = None
    vmhost: str = None


class MachineLogModel(BaseModel):
    id: int = None
    physical: Optional[str] = None
    logical: Optional[str] = None
    stamp: Optional[str] = Union[datetime, date]
    type: Optional[str] = None
    comment: Optional[str] = None
    service_queue_id: Optional[int] = None
    user: Optional[str] = None


class MachineModelWithLog(MachineModel):
    log: List[MachineLogModel] = []
    ipmi: IPv4Address = None


class inDedicatedConfugrationStatuses(BaseModel):
    class statusenum(Enum):
        active = 'active'
        inactive = 'inactive'
        hidden = 'hidden'

    status: statusenum
