from .core import MachineServiceBase, PROD_API_URL
from functools import lru_cache


class Zabbix():
    def __init__(self, machine=None, client=None, log_level='WARN', url=PROD_API_URL):
        self.machine = machine
        self.client = client or MachineServiceBase(
            log_level=log_level, url=url)

    # @lru_cache()
    def get(self):
        return self.client._get(path='/zabbix/%s' % self.machine)

    # @lru_cache()
    def dashboard(self):
        """
        Return URL for this machine's Zabbix dashboard
        """
        return self.client._get(path='/zabbix/%s/dashboard' % self.machine)

    # @lru_cache()
    def latest_data(self):
        """
        Return URL for this machine's Zabbix latest data page
        """
        return self.client._get(path='/zabbix/%s/latest_data' % self.machine)

    # @lru_cache()
    def edit(self):
        """
        Return URL for this machine's Zabbix edit page
        """
        return self.client._get(path='/zabbix/%s/edit' % self.machine)

    # @lru_cache()
    def cpu(self):
        """
        Get CPU metrics for this machine from Zabbix
        """
        return self.client._get(path='/zabbix/%s/cpu' % self.machine)

    # @lru_cache()
    def memory(self):
        """
        Get memory metrics for this machine from Zabbix
        """
        return self.client._get(path='/zabbix/%s/memory' % self.machine)

    # @lru_cache()
    def problems(self):
        """
        Returns a list of Zabbix problems
        """
        return self.client._get(path='/zabbix/%s/problems' % self.machine)

    # @lru_cache()
    def proxy(self):
        """
        Returns the proxy this machine is assigned to
        """
        return self.client._get(path='/zabbix/%s/proxy' % self.machine)

    # @lru_cache()
    def down(self):
        """
        Disable host in zabbix
        """
        return self.client._get(path='/zabbix/%s/disable' % self.machine)

    # @lru_cache()
    def disable(self):
        """
        Disable host in zabbix
        """
        return self.client._get(path='/zabbix/%s/disable' % self.machine)

    # @lru_cache()
    def up(self):
        """
        Enable host in zabbix
        """
        return self.client._get(path='/zabbix/%s/enable' % self.machine)

    # @lru_cache()
    def enable(self):
        """
        Enable host in zabbix
        """
        return self.client._get(path='/zabbix/%s/enable' % self.machine)
