from cement import Controller, ex
from towerlib import Tower
from awxkit.cli import run
import sys


def get_tower_client(controller):
    controller.app.require_creds()
    return Tower(controller.app.config.get('ndn', 'ansible_awx_api').get(controller.app.pargs.environment),
                 controller.app.username, controller.app.password, secure=True)


class AnsibleAWX(Controller):
    class Meta:
        label = 'awx'
        stacked_type = 'nested'
        stacked_on = 'ansible'
        help = 'Ansible awx (job scheduling). This is just a wrapper around awxkit. To pass additional flag, use "-- --help" for example.'
        arguments = [
            (['--environment'],
             {'help': 'AWX API environment (defined in config)',
              'action': 'store',
              'choices': ['prod', 'dev'],
              'default': 'prod'}),
            (['--api-username'],
             {'help': 'AWX API username (if not supplied, will try to use cached creds)',
              'action': 'store',
              'default': None}),
            (['--api-password'],
             {'help': 'AWX API password (if not supplied, will try to use cached creds)',
              'action': 'store',
              'default': None}),
            (['-f', '--format'],
             {'help': 'AWX output format (human, json, yaml)',
              'action': 'store',
              'default': 'json'}),
            (['awxargs'],
             {'action': 'store',
              'nargs': '+'}),
        ]

    @ex(help='Execute an AWX command')
    def _default(self):
        if not self.app.pargs.api_username or not self.app.pargs.api_password:
            self.app.require_creds(
                self.app.pargs.api_username,
                self.app.pargs.api_password)

        awx_host = self.app.config.get(
            'ndn', 'ansible_awx_api').get(self.app.pargs.environment)
        awx_username = self.app.pargs.api_username or self.app.username
        awx_password = self.app.pargs.api_password or self.app.password

        awx_cmd = ['awx', '--conf.host', awx_host, '--conf.username', awx_username,
                   '--conf.password', awx_password, '-f', self.app.pargs.format] + self.app.pargs.awxargs

        # this hack is needed to make the awxkit happy when running through python-cement
        # i'm sure this is fine and will in no way ever cause problems later >.>
        sys.argv = awx_cmd

        run(argv=awx_cmd)
