from cement import Controller, ex
import pprint
from ansible_client.base import AnsibleClient


class AnsibleInventory(Controller):
    class Meta:
        label = 'inventory'
        stacked_type = 'nested'
        stacked_on = 'ansible'
        help = 'Ansible inventory refresh and show'

    @ex(help='Schedule an ansible inventory refresh')
    def refresh(self):
        self.app.require_token()
        schedule = AnsibleClient().inventory_refresh()
        if schedule:
            print(schedule)
        else:
            self.app.log.warning(
                "Failed to schedule ansible inventory refresh")
            self.app.exit_code = 1
            self.app.close()

    @ex(
        help='Get status information for an ansible inventory refresh task UUID',
        arguments=[
            (['uuid'],
             {'help': 'UUID of ansible inventory refresh task',
              'action': 'store'})])
    def status(self):
        self.app.require_token()
        status = AnsibleClient().inventory_refresh_status(self.app.pargs.uuid)
        if status:
            pprint.pprint(status)
        else:
            self.app.log.warning(
                "Failed to fetch ansible inventory task status for UUID %s" % self.app.pargs.uuid)
            self.app.exit_code = 1
            self.app.close()

    @ex(
        help='JSON representation of the ansible inventory',
        arguments=[
            (['-f', '--format'],
             {'help': 'Format of console output of ansible inventory (raw/json)',
              'action': 'store',
              'default': 'json'})
        ]
    )
    def show(self):
        self.app.require_token()
        inventory = AnsibleClient().get_inventory()
        if inventory:
            if self.app.pargs.format == 'json':
                pprint.pprint(inventory)
            else:
                print(inventory)
        else:
            self.app.log.warning("Failed to fetch ansible inventory")
            self.app.exit_code = 1
            self.app.close()

    @ex(help='Is the ansible inventor fresh?')
    def freshness(self):
        self.app.require_token()
        freshness = AnsibleClient().inventory_freshness()
        if freshness:
            self.app.log.debug(
                'Returning ansible freshness details: %s' % freshness)
            self.app.render(
                {'data': freshness, 'args': self.app.pargs.__dict__}, 'ansible/freshness.jinja2')
        else:
            self.app.log.warning(
                "Unable to check ansible inventory freshness (rerun with -d flag)")
            self.app.exit_code = 1
            self.app.close()

    @ex(help='Schedule an ansible inventory update for a single host',
        arguments=[
            (['host'],
             {'help': 'Machine name of host for which to schedule an immediate inventory refresh',
              'action': 'store',
              'default': None})
        ]
        )
    def host_update(self):
        self.app.require_token()
        schedule = AnsibleClient().inventory_host_refresh(self.app.pargs.host)
        if schedule:
            print(schedule)
        else:
            self.app.log.warning(
                f"Failed to schedule ansible inventory refresh for host {self.app.pargs.host}")

    @ex(help='Get the update status of an update request for a single host',
        arguments=[
            (['uuid'],
             {'help': 'UUID of ansible inventory single-host refresh task',
              'action': 'store'})
        ]
        )
    def host_update_status(self):
        self.app.require_token()
        status = AnsibleClient().inventory_object_refresh_status(self.app.pargs.uuid)
        if status:
            pprint.pprint(status)
        else:
            self.app.log.warning(
                "Failed to fetch ansible inventory host refresh task status for UUID %s" % self.app.pargs.uuid)
            self.app.exit_code = 1

    @ex(help='Schedule an ansible inventory update for a single group/cluster',
        arguments=[
            (['group'],
             {'help': 'Cluster/group name of host for which to schedule an immediate inventory refresh',
              'action': 'store',
              'default': None})
        ]
        )
    def group_update(self):
        self.app.require_token()
        schedule = AnsibleClient().inventory_group_refresh(self.app.pargs.group)
        if schedule:
            print(schedule)
        else:
            self.app.log.warning(
                f"Failed to schedule ansible inventory refresh for group {self.app.pargs.group}")

    @ex(help='Get the update status of an update request for a single group/cluster',
        arguments=[
            (['uuid'],
             {'help': 'UUID of ansible inventory single-group/cluster refresh task',
              'action': 'store'})
        ]
        )
    def group_update_status(self):
        self.app.require_token()
        status = AnsibleClient().inventory_object_refresh_status(self.app.pargs.uuid)
        if status:
            pprint.pprint(status)
        else:
            self.app.log.warning(
                "Failed to fetch ansible inventory group refresh task status for UUID %s" % self.app.pargs.uuid)
            self.app.exit_code = 1
