from cement import Controller, ex
from getpass import getpass
import requests


class Apis(Controller):
    class Meta:
        label = 'apis'
        stacked_type = 'nested'
        stacked_on = 'base'
        help = 'NDN API interaction'

    @ex(
        help='Login to DH APIs (hint: LDAP creds)',
        arguments=[
            (['-u', '--username'],
             {'help': 'Login username (hint: LDAP)',
              'action': 'store',
              'dest': 'username',
              'default': None}),
            (['-p', '--password'],
             {'help': 'Login password (hint: LDAP)',
              'action': 'store',
              'dest': 'password',
              'default': None})
        ]
    )
    def login(self):
        username = self.app.pargs.username or input('Username: ')
        password = self.app.pargs.password or getpass()
        self.app.log.debug("Sending auth login to %s" %
                           self.app.config.get('ndn', 'auth_api'))
        auth_resp = requests.post(
            self.app.config.get('ndn', 'auth_api'),
            headers={"Content-type": "application/json"},
            json={"username": username, "password": password})
        if auth_resp.status_code == 200:
            self.app.log.debug("Authentication successful")
            token = auth_resp.json()['access_token']
            self.app.log.debug("Saving token to %s" % self.app.token_file)
            with open(self.app.token_file, 'w') as f:
                f.write(token)
        elif auth_resp.status_code == 403:
            self.app.log.debug("Authentication failed: %s" % auth_resp.content)
            self.app.log.error("Username/password incorrect")
            self.app.exit_code = 1
        else:
            self.app.log.debug("Authentication API failure: %s" %
                               auth_resp.content)
            self.app.log.error("Authentication API failure:")
            self.app.log.error(auth_resp.content)
            self.app.exit_code = 1

    @ex(
        help='List the registered APIs. If you notice an API missing from this list, ensure it is registered in Backstage.',
        arguments=[
            (['-n', '--name'],
             {'help': 'Filter the list based on the name of the project (substring matching)',
              'action': 'store',
              'default': None}),
        ]
    )
    def list(self):
        api_list = self.app.get_url(
            self.app.config.get('ndn', 'backstage_api') +
            '/catalog/entities/by-query',
            params={'filter': 'kind=Component'})

        # sort the list by name to pretify the output
        sorted_list = sorted(
            api_list['items'], key=lambda d: d['metadata']['name'])

        if self.app.pargs.name:
            sorted_list = [
                service for service in sorted_list if self.app.pargs.name in service['metadata']['name']]

        self.app.render(
            {'args': self.app.pargs, 'data': sorted_list}, 'apis/catalog.jinja2')
