
from cement import Controller
from cement.utils.version import get_version_banner
from ..core.version import get_version

VERSION_BANNER = """
DH CLI for operations %s
%s
""" % (get_version(), get_version_banner())


class Base(Controller):
    class Meta:
        label = 'base'

        # text displayed at the top of --help output
        description = 'DH CLI for operations'

        # text displayed at the bottom of --help output
        epilog = 'Usage: ndn <command>'

        # controller level arguments. ex: 'ndn --version'
        arguments = [
            # add a version banner
            (['-v', '--version'],
             {'action': 'version',
                'version': VERSION_BANNER}),
            (['--no-colorize'],
             {'action': 'store_false',
                'dest': 'colorize',
                'default': True,
                'help': 'Disable colorized output'}),
            (['--enable-plugin'],
             {'help': 'Specific plugin to load, even though it is not in the config file. Useful for developing new plugins',
              'action': 'store'}),
            (['--output-handler'],
             {'help': 'Swap between different output formats',
              'dest': 'output_handler',
              'default': None,
              'choices': ['jinja2', 'json', 'tabulate', 'yaml'],
              'action': 'store'})
        ]

    def _default(self):
        """Default action if no sub-command is passed."""

        self.app.args.print_help()
