from cement import Controller, ex
from cluster_client.cluster import Cluster as ClusterAPI
from cluster_client.core import ClusterBase


class Cluster(Controller):
    class Meta:
        label = 'cluster'
        stacked_type = 'nested'
        stacked_on = 'base'
        help = 'Interface for DH Cluster object (dh-cluster)'
        arguments = [
            (['cluster'],
             {'action': 'store',
              'nargs': '+'}),
            (['-v', '--verbose'],
             {'help': "Increase verbosity",
              'action': 'store_true'}),
            (['-n', '--newline'],
             {'help': "Newline delimited output (default is space)",
              'action': 'store_true'}),
            (['-a', '--all'],
             {'help': "Show all cluster members, not just members marked up",
              'action': 'store_true',
              'dest': 'all_machines'}),
            (['-p', '--params'],
             {'help': "Print cluster params, but don't list cluster members",
              'action': 'store_true'}),
            (['-P', '--physical'],
             {'help': "Translate logical machines into associated physical machines",
              'action': 'store_true'}),
            (['-l', '--logical'],
             {'help': "Translate physical machines into associated logical machines",
              'action': 'store_true'}),
            (['-L', '--location'],
             {'help': "Prints all machines by data center location",
              'action': 'store_true'}),
            (['-i', '--intersection'],
             {'help': "Display members that exist in each cluster",
              'action': 'store_true'}),
            (['-e', '--exclude'],
             {'help': "Exclude members of this cluster. Can be passed multiple times",
              'action': 'append'}),
            (['-s', '--search'],
             {'help': "Displays names of clusters like search term",
              'action': 'store_true'}),
            (['-d', '--set-description'],
             {'help': "Set the cluster(s) description",
              'action': 'store'}),
            (['-c', '--set-class'],
             {'help': "Set the cluster(s) class",
              'action': 'store'}),
            (['-C', '--create'],
             {'help': "Create the cluster(s)",
              'action': 'store_true'}),
            (['-D', '--destroy'],
             {'help': "Destroy the cluster (only accepts a single cluster)",
              'action': 'store_true'}),
            (['-A', '--add-machine'],
             {'help': "Add given machines (space separated) to the cluster(s)",
              'action': 'store',
              'nargs': '+'}),
            (['-R', '--remove-machine'],
             {'help': "Remove given machines (space separated) from the cluster(s)",
              'action': 'store',
              'nargs': '+'}),
            (['-q', '--add-ansible-child'],
             {'help': "(Ansible) Designate a cluster or clusters children (space separated)",
              'action': 'store',
              'nargs': '+'}),
            (['-x', '--remove-ansible-child'],
             {'help': "(Ansible) Remove parent-child relationships (space separated)",
              'action': 'store',
              'nargs': '+'}),
            (['-r', '--recurse'],
             {'help': "(Ansible) Also show machines in any child clusters, recursively",
              'action': 'store_true'}),
            (['--parent'],
             {'help': "(Ansible) Show the parents of the given cluster(s) (recursive)",
              'action': 'store_true'})
        ]

    @ex(help="Display information about a cluster")
    def info(self):
        """
        Fetch and display cluster details
        """
        info = ClusterAPI(
            {'cluster': self.app.pargs.cluster[0]},
            url=self.app.config.get('ndn', 'cluster_api')).get(
            params=self.app.pargs.params).dict(by_alias=True)
        self.app.render({'data': info}, 'cluster/cluster.jinja2')

    @ex(help="List machines that belong to a cluster")
    def machines(self):
        """
        List machines that belong to a cluster
        """

        client = ClusterBase()

        for cluster in self.app.pargs.cluster:
            if self.app.pargs.location:
                machines = ClusterAPI(
                    {'cluster': cluster, 'client': client},
                    url=self.app.config.get('ndn', 'cluster_api')).machines(
                    location=self.app.pargs.location, all_machines=self.app.pargs.all_machines)
            else:
                machines = ClusterAPI(
                    {'cluster': cluster, 'client': client},
                    url=self.app.config.get('ndn', 'cluster_api')).machines(
                    all_machines=self.app.pargs.all_machines)

        self.app.render({'data': {'args': self.app.pargs,
                        'machines': machines}}, 'cluster/machines.jinja2')
