# NOTE: this subcommand was written to talk to the python domain-service. that
# service no longer exists. this subcommand needs to be updated to talk to the
# API3 REST interface
from cement import Controller, ex
import subprocess


class Domain(Controller):
    class Meta:
        label = 'domain'
        stacked_type = 'nested'
        stacked_on = 'base'
        help = 'Interface for DH Domains (dh-domain)'
        arguments = [
            (['domain'],
             {'help': 'Domain name (i.e. dreamhost.com)',
              'action': 'store'})
        ]

    @ex(help='Add a new domain to the DB')
    def add(self):
        pass

    @ex(help='Display domain information (synonym for "print")')
    def info(self):
        self.print_or_info()

    @ex(help='Display domain information (synonym for "info")')
    def print(self):
        self.print_or_info()

    def print_or_info(self):
        self.app.require_token()
        domain_details = self.app.get_url(
            self.app.config.get('ndn', 'domain_api') +
            '/' + self.app.pargs.domain,
            params={'extended': True},
            headers=self.app.AUTH_HEADER)
        if domain_details:
            domain_details['args'] = self.app.pargs.__dict__
            self.app.render({'data': domain_details}, 'domain/print.jinja2')

    @ex(help='SSH into the domain',
        arguments=[
            (['-u', '--user'],
             {'help': 'Target SSH user on the remote system',
              'default': False,
              'action': 'store'}),
            (['-p', '--private-key'],
             {'help': 'File path to SSH private key to access user on the remote system',
              'dest': 'private_key',
              'default': False,
              'action': 'store'}),
        ]
        )
    def enter(self):
        machine = self.app.get_url(
            self.app.config.get('ndn', 'domain_api') +
            '/machine/' + self.app.pargs.domain,
            headers=self.app.AUTH_HEADER)
        if not machine:
            self.app.log.error(
                "Unable to find machine for domain '%s'" % self.app.pargs.domain)
            self.app.exit_code = 1
            self.app.close()

        ip = machine['ip']
        if not ip:
            self.app.log.error(
                "Home machine '%s' does not have an IP assigned" % machine['machine'])
            self.app.exit_code = 1
            self.app.close()

        # setup and call our ssh command
        command = "/usr/bin/ssh"
        if self.app.pargs.user:
            command = command + ' -l ' + self.app.pargs.user
        if self.app.pargs.private_key:
            command = command + ' -i ' + self.app.pargs.private_key
        command = command + ' ' + machine['ip']
        self.app.log.debug("executing command: %s" % command)
        subprocess.call(command, shell=True)

    @ex(help='Set domain to inactive')
    def inactive(self):
        inactive = self.app.post_json_url(
            self.app.config.get('ndn', 'domain_api') +
            '/inactive/%s' % self.app.pargs.domain,
            headers=self.app.AUTH_HEADER)
        if inactive:
            for service in inactive:
                print("Deactivating service: %s" % service)
        else:
            self.log.error("Domain %s failed to park" % self.app.pargs.domain)
            self.app.exit_code = 1
            self.app.close()

    @ex(help='Set domain to parked')
    def parked(self):
        park = self.app.post_json_url(
            self.app.config.get('ndn', 'domain_api') +
            '/park/%s' % self.app.pargs.domain,
            headers=self.app.AUTH_HEADER)
        if park:
            print("Parking domain: %s" % self.app.pargs.domain)
        else:
            self.log.error("Domain %s failed to park" % self.app.pargs.domain)
            self.app.exit_code = 1
            self.app.close()

    @ex(help='Run a Service Control to configure the domain',
        arguments=[
            (['-c', '--channel'],
             {'help': 'Schedule the domain config in a specific Service Control channel',
              'default': False,
              'action': 'store'})
        ]
        )
    def config(self):
        # Fetch DSes for this domain
        domain_services = self.app.get_url(
            self.app.config.get('ndn', 'domain_api') +
            '/services/' + self.app.pargs.domain,
            headers=self.app.AUTH_HEADER)
        if not domain_services:
            self.app.log.warning(
                'Unable to find any domain services for domain %"s"' % self.app.pargs.domain)
            self.app.exit_code = 1
            self.app.close()

        # Get the http/s service for this domain
        service = None
        for ds in domain_services:
            if ds['type'] in ['http', 'https'] and ds['status'] in ['active', 'changed']:
                # TODO need to check for `skip-expire` param
                service = ds
                break
        if not service:
            self.app.log.warning(
                'Unable to find any http/s DS for domain "%s"' % self.app.pargs.domain)
            self.app.exit_code = 1
            self.app.close()

        # send sc schedule request to the API
        params = {
            'home': service['home'],
            'service': service['service'],
            'command': 'config',
            'channel': self.app.pargs.channel or 'default'
        }
        schedule = self.app.put_json_url(
            self.app.config.get('ndn', 'servicectl_api') + '/jobs/schedule',
            headers=self.app.AUTH_HEADER,
            json=params)

        if not schedule:
            self.app.log.warning(
                'Failed to schedule config for domain "%s"' % self.app.pargs.domain)
            self.app.exit_code = 1
            self.app.close()

        # add CLI args to the output
        schedule['args'] = self.app.pargs.__dict__
        self.app.render({'data': schedule}, 'servicectl/job.jinja2')
