from cement import Controller, ex
import pprint


class Lighthouse(Controller):
    class Meta:
        label = 'lighthouse'
        stacked_type = 'nested'
        stacked_on = 'base'
        help = 'Trigger and check the status of lighthouse jobs'
        arguments = [
            (['domain'],
             {'help': 'Domain name (i.e. example.com)',
              'action': 'store'}),
        ]

    @ex(help='Schedule a lighthouse run for a given domain')
    def schedule(self):
        schedule = self.app.post_json_url(
            self.app.config.get('ndn', 'lighthouse_api') +
            '/' + self.app.pargs.domain,
            headers=self.app.AUTH_HEADER, json={})
        if schedule:
            print(schedule)
        else:
            self.app.log.warning(
                "Failed to schedule lighthouse run for domain %s" % self.app.pargs.domain)
            self.app.exit_code = 1
            self.app.close()

    @ex(help='Get status information for a lighthouse task UUID')
    def status(self):
        status = self.app.get_url(
            self.app.config.get('ndn', 'lighthouse_api') +
            '/fetch_task_status/' + self.app.pargs.domain,
            headers=self.app.AUTH_HEADER)
        if status:
            pprint.pprint(status)
        else:
            self.app.log.warning(
                "Failed to fetch lighthouse status for UUID %s" % self.app.pargs.domain)
            self.app.exit_code = 1
            self.app.close()
