from cement import Controller, ex
from os import remove, environ


class Login(Controller):
    class Meta:
        label = 'login'
        stacked_type = 'nested'
        stacked_on = 'base'
        help = 'NDN API and vault token authentication'
        arguments = [
            (['-u', '--username'],
             {'help': 'Login username (hint: LDAP, will look at env var NDN_LDAP_USERNAME)',
             'action': 'store',
              'dest': 'username',
              'default': None}),
            (['-p', '--password'],
             {'help': 'Login password (hint: LDAP, will look at env var NDN_LDAP_PASSWORD)',
              'action': 'store',
              'dest': 'password',
              'default': None})
        ]

    @ex(
        help='Login to DH APIs and vault (hint: LDAP creds)',
    )
    def _default(self):
        self.app.username = self.app.pargs.username or environ.get(
            'NDN_LDAP_USERNAME', None)
        self.app.password = self.app.pargs.password or environ.get(
            'NDN_LDAP_PASSWORD', None)
        self.app.require_creds(self.app.username, self.app.password)
        self.app.api_login(self.app.username, self.app.password)
        self.app.vault_login(self.app.username, self.app.password)

    @ex(
        help='Clear (delete) the locally cached credentials',
    )
    def clear_cache(self):
        for cache_file in [self.app.jwt_token_file, self.app.vault_token_file, self.app.creds_cache_file]:
            self.app.log.debug(
                f"Removing cached credentials stored in file {cache_file}")
            remove(cache_file)
        print("Successfully removed cached credentials")
