from cement import Controller, ex
from os import listdir, path
from getpass import getuser
import ruamel.yaml


class Plugin(Controller):
    class Meta:
        label = 'plugin'
        stacked_type = 'nested'
        stacked_on = 'base'
        help = 'Interfaces for ndn CLI plugin management'

    def _set_plugin(self, plugin, enabled=True):
        conf_file = path.expanduser('~%s/.config/ndn/ndn.yml' % getuser())
        yaml = ruamel.yaml.YAML()
        with open(conf_file, 'r') as cfile:
            conf = yaml.load(cfile.read())

        conf.update({f'plugin.{plugin}': {'enabled': enabled}})

        with open(conf_file, 'w') as cfile:
            yaml.dump(conf, cfile)

    @ex(help='List the plugins that we know about/can find')
    def list(self):
        plugs = {}
        for plugin in self.app.plugin.load_dirs:
            self.app.log.debug(f'searching plugin dir {plugin}')
            try:
                for plug in listdir(plugin):
                    self.app.log.debug(f'  found plugin {plug}')
                    plugs[plug] = {'path': f'{plugin}/{plug}',
                                   'status': 'Enabled' if plug in self.app.plugin.get_loaded_plugins() else 'Disabled'}
            except FileNotFoundError as err:
                self.app.log.debug(f'  plugin dir not found: {err}')
                continue

        print('\n'.join(
            [f'{plugin}: {plugs[plugin]["path"]} [{plugs[plugin]["status"]}]' for plugin in plugs]))

    @ex(help="List the plugin directories we're configured to know about")
    def dirs(self):
        print('\n'.join(self.app.plugin.load_dirs))

    @ex(
        help='Enable a specific plugin by updating the config file',
        arguments=[
            (['plugin'],
             {'help': 'The name of the plugin to enable'})])
    def enable(self):
        self._set_plugin(self.app.pargs.plugin, True)

    @ex(
        help='Disable a specific plugin by updating the config file',
        arguments=[
            (['plugin'],
             {'help': 'The name of the plugin to disable'})])
    def disable(self):
        self._set_plugin(self.app.pargs.plugin, False)
