from cement import Controller, ex
from pprint import pprint
from servicectl_client.channels import Channel


class ServiceCtlChannels(Controller):
    class Meta:
        label = 'channels'
        stacked_type = 'nested'
        stacked_on = 'servicectl'
        help = 'List Service Control channels'

    @ex(help='List valid Service Control channels')
    def _default(self):
        self.app.require_token()
        self.app.render({'data': set(Channel().list())},
                        'servicectl/channels.jinja2')

    @ex(help='List valid Service Control channels')
    def list(self):
        self.app.require_token()
        self.app.render({'data': set(Channel().list())},
                        'servicectl/channels.jinja2')

    @ex(
        help='Add a new Service Control channel',
        arguments=[
            (['-c', '--channel'],
             {'help': 'Channel name',
              'default': None,
              'action': 'store',
              'required': True}),
            (['-m', '--max-jobs'],
             {'help': 'Maximum concurrent jobs allowed to be run in this channel (default: 5)',
              'default': 5,
              'action': 'store'}),
            (['-r', '--retries'],
             {'help': 'Number of retries Service Control will attempt for jobs in this channel if the job is not successful (default: 0)',
              'default': 0,
              'action': 'store'}),
            (['-R', '--realtime'],
             {'help': 'Ignore load and execute jobs as quickly as possible (default: False)',
              'default': False,
              'action': 'store_true'}),
        ])
    def add(self):
        self.app.require_token()
        channel = Channel({
            'channel': self.app.pargs.channel,
            'max_jobs': self.app.pargs.max_jobs,
            'retries': self.app.pargs.retries or 0,
            'realtime': 'true' if self.app.pargs.realtime else 'false'})
        channel.add()
        print('Successfully created channel "%s": %s' %
              (self.app.pargs.channel, channel.channel.model_dump()))

    @ex(
        help='Delete a Service Control channel',
        arguments=[
            (['channel'],
             {'help': 'Channel name or ID',
              'default': None,
              'action': 'store'}),
        ])
    def delete(self):
        self.app.require_token()
        channel = Channel({'channel': self.app.pargs.channel})
        channel.get()
        old_channel = channel.delete()
        if not old_channel:
            print('Failed to delete channel with name/id "%s". Re-run with -d for more information.' %
                  self.app.pargs.channel)
        else:
            print('Successfully deleted channel with name/id "%s": %s' %
                  (self.app.pargs.channel, old_channel))

    @ex(
        help='Fetch details about a Service Control channel',
        arguments=[
            (['channel'],
             {'help': 'Channel name or ID',
              'default': None,
              'action': 'store'}),
        ])
    def get(self):
        self.app.require_token()
        channel = Channel({'channel': self.app.pargs.channel})
        channel.get()
        pprint(channel.channel.model_dump())

    @ex(
        help='Edit a Service Queue channel',
        arguments=[
            (['channel'],
             {'help': 'Channel name or ID',
              'default': None,
              'action': 'store'}),
            (['-c', '--channel_name'],
             {'help': 'Channel name',
              'default': None,
              'action': 'store'}),
            (['-m', '--max_jobs'],
             {'help': 'Maximum number of concurrent jobs for this channel',
              'default': None,
              'action': 'store'}),
            (['-r', '--retries'],
             {'help': 'Number of retries for jobs in this channel',
              'default': None,
              'action': 'store'}),
            (['-R', '--realtime'],
             {'help': '"realtime"',
              'default': None,
              'action': 'store'}),
        ])
    def edit(self):
        self.app.require_token()
        chan = Channel({'channel': self.app.pargs.channel})
        chan.get()

        chan.channel.channel = str(
            getattr(self.app.pargs, 'channel_name') or chan.channel.channel)
        chan.channel.max_jobs = int(
            getattr(self.app.pargs, 'max_jobs') or chan.channel.max_jobs)
        chan.channel.retries = int(
            getattr(self.app.pargs, 'retries') or chan.channel.retries)
        chan.channel.realtime = str(
            getattr(self.app.pargs, 'realtime') or chan.channel.realtime)

        chan.save()
        pprint(chan.channel.model_dump())
