from cement import Controller, ex
from pprint import pprint
from servicectl_client.health import Health
import time


class ServiceCtlHealth(Controller):
    class Meta:
        label = 'health'
        stacked_type = 'nested'
        stacked_on = 'servicectl'
        help = 'Service Control health checks'

    @ex(help='Report API timings (http, db read, db write). This writes to the DB, so do not abuse it.')
    def _default(self):
        self.app.require_token()

        api_start_time = time.time()
        api_response = Health().pingz()
        api_elapsed_time = round(time.time() - api_start_time, 4)
        print(f"API response time: {api_elapsed_time}")

        db_ping_start_time = time.time()
        db_ping_response = Health().dbpingz()
        db_ping_elapsed_time = round(time.time() - db_ping_start_time, 4)
        print(f"DB ping response time: {db_ping_elapsed_time}")

        db_full_start_time = time.time()
        db_full_response = Health().dbthroughput()
        db_full_elapsed_time = round(time.time() - db_full_start_time, 4)
        print(f"DB throughput response time: {db_full_elapsed_time}")

    @ex(help='Validate basic API functionality')
    def pingz(self):
        self.app.require_token()

        start_time = time.time()
        response = Health().pingz()
        elapsed_time = round(time.time() - start_time, 4)

        pprint({'response': response,
                'API_response_timing': f"{elapsed_time} ms"})

    @ex(help='Validate the API can access the DB backend')
    def db_pingz(self):
        self.app.require_token()

        start_time = time.time()
        response = Health().dbpingz()
        elapsed_time = round(time.time() - start_time, 4)

        pprint({'response': response,
                'API_response_timing': f"{elapsed_time} ms"})

    @ex(help='Validate the API can write/read/delete from the DB backend. This '
             'does real reads/writes, so do not abuse it.')
    def db_throughput(self):
        self.app.require_token()

        start_time = time.time()
        response = Health().dbthroughput()
        elapsed_time = round(time.time() - start_time, 4)

        pprint({'response': response,
                'API_response_timing': f"{elapsed_time} ms"})
