from cement import Controller, ex
import hvac
import pprint


UANDPARG = [
    (['-u', '--vault_username'],
     {'help': 'Vault username (LDAP)',
      'action': 'store',
      'default': None}),
    (['-p', '--vault_password'],
     {'help': 'Vault password (LDAP)',
      'action': 'store',
      'default': None})
]


class Vault(Controller):
    class Meta:
        label = 'vault'
        stacked_type = 'nested'
        stacked_on = 'base'
        help = 'Read information from NDN Hashicorp-Vault secret engine (for more options, use the vault client directly)'

    @ex(help='Login to vault, retrieving and storing a new token',
        arguments=UANDPARG)
    def login(self):
        self.app.vault_login(self.app.pargs.vault_username,
                             self.app.pargs.vault_password)

    @ex(help='Returns a list of items in "secret" storage engine',
        arguments=UANDPARG)
    def list(self):
        client = self.app.vault_login(
            self.app.pargs.vault_username, self.app.pargs.vault_password)

        def _get_secrets(path):
            print('_get_secrets(%s)' % path)
            secrets = {}
            for this_path in client.secrets.kv.v1.list_secrets(path=path)['data']['keys']:
                secrets[path] = _get_secrets(this_path)
            return secrets

        secrets = {'/': []}
        for path in client.secrets.kv.v1.list_secrets(path='/')['data']['keys']:
            try:
                secrets[path] = _get_secrets(path)
            except hvac.exceptions.InvalidPath:
                secrets['/'].append(path)

    @ex(
        help='Returns the contents of a vault kv secret',
        arguments=UANDPARG + [
            (['path'],
             {'help': 'Vault secret path'}
             )])
    def get(self):
        client = self.app.vault_login(
            self.app.pargs.vault_username, self.app.pargs.vault_password)

        content = client.secrets.kv.v1.read_secret(
            path=self.app.pargs.path, mount_point='secret')['data']

        pprint.pprint(content)
