from cement import Controller, ex


class Wordpress(Controller):
    class Meta:
        label = 'wordpress'
        stacked_type = 'nested'
        stacked_on = 'base'
        help = 'Interfaces for getting information about WordPress installs'

    @ex(
        help='Returns metadata information for a given WebApplicationInstall ID',
        arguments=[
            (['wai'],
             {'help': 'WebApplicationInstall ID or domain name',
              'action': 'store'})])
    def get_metadata(self):
        try:
            # is this an int? if so, it's probably an install ID
            wai_type = 'by_wai' if int(self.app.pargs.wai) else 'by_domain'
        except ValueError:
            # couldn't cast wai to an int, which means it's probably a domain name
            wai_type = 'by_domain'

        result = self.app.get_url(
            self.app.config.get('ndn', 'wordpress_api') +
            '/metadata/%s/%s' % (wai_type, self.app.pargs.wai),
            headers=self.app.AUTH_HEADER)
        if not result:
            self.app.log.warning(
                'Unable to locate metadata for WebApplicationInstall ID %s' % self.app.pargs.wai)
            self.app.exit_code = 1
            self.app.close()
        else:
            self.app.render({'data': {'args': self.app.pargs.__dict__,
                            'result': result}}, 'wordpress/metadata.jinja2')

    @ex(
        help='Schedule a refresh of metadata information for a given WebApplicationInstall ID',
        arguments=[
            (['wai'],
             {'help': 'WebApplicationInstall ID',
              'action': 'store'})])
    def refresh_metadata(self):
        schedule = self.app.get_url(
            self.app.config.get('ndn', 'wordpress_api') +
            '/metadata/refresh/%s' % self.app.pargs.wai,
            headers=self.app.AUTH_HEADER)
        if schedule:
            print(schedule)
        else:
            self.app.log.warning(
                "Failed to schedule a metadata refresh run for WebApplicationInstall ID %s" % self.app.pargs.wai)
            self.app.exit_code = 1
            self.app.close()

    @ex(
        help='Fetch metadata refresh task information',
        arguments=[
            (['uuid'],
             {'help': 'Refresh task UUID',
              'action': 'store'})])
    def metadata_task(self):
        status = self.app.get_url(
            self.app.config.get('ndn', 'wordpress_api') +
            '/fetch_task_status/' + self.app.pargs.uuid,
            headers=self.app.AUTH_HEADER)
        if status:
            pprint.pprint(status)
        else:
            self.app.log.warning(
                "Failed to fetch metadata task status for UUID %s" % self.app.pargs.uuid)
            self.app.exit_code = 1
            self.app.close()
