import os
from .vault import vault_login


class NdnSecureAuth():
    def __init__(self, name):
        self.ndnsecure_path = os.environ.get('NDNSECURE_PATH', '/etc/ndn')
        # name of auth file
        self.name = name
        self.ndnsecure_file = os.path.join(self.ndnsecure_path, 'auth', name)

    # returns auth file contents (each line) as list
    def get(self, split=True):
        # load from env vars
        # '/', '.', and '-' are invalid env var names, so we do some swapping to make them valid
        result = os.getenv("NDN_AUTH_%s" % self.name.replace(
            '/', '___').replace('.', '_').replace('-', '_'), None)
        if result:  # if we fetched from env var, split based on "\n" to match file-based secrets
            result = result.split('\n')
        # if the env var wasn't set, check vault/file
        if not result:
            if os.environ.get('NDNSECURE_USE_VAULT', False):
                vclient = vault_login()
                result = vclient.secrets.kv.v1.read_secret(
                    path='auth/%s' % self.name, mount_point='secret')['data']['content'].split('\n')
            else:
                with open(self.ndnsecure_file, 'r') as secretfile:
                    result = [line.rstrip() for line in secretfile]
        if not split:
            result = '\n'.join(result)

        return result
