import os
import hvac
import warnings
from urllib3.connectionpool import InsecureRequestWarning
from getpass import getuser


# TODO once we verify ssl for vault again, remove this
warnings.filterwarnings("ignore", category=InsecureRequestWarning)


def vault_login(url=None, username=None, password=None, verify=False):
    """Login to Vault and return an authenticated client"""
    if url is None:
        url = os.environ.get(
            'VAULT_URL', 'https://active-vault.dh-int.com')
    if username is None:
        username = os.environ.get('VAULT_USERNAME', 'vault_user')
    if password is None:
        password = os.environ.get('VAULT_PASSWORD', 'vault_pass')
    user_token_file = os.path.expanduser('~%s/.vault-token' % getuser())
    client = hvac.Client(url=url, verify=verify)
    if os.path.isfile(user_token_file):
        with open(user_token_file, 'r') as vtoken:
            client.token = vtoken.read()
    try:
        authed = client.is_authenticated()
    except ValueError:
        # if is_authenticated() fails, that probably means the token is bad, so unset it
        client.token = None
        authed = False
    if not authed:
        client.auth.ldap.login(username=username, password=password)
        try:
            # we use getuser here to make sure we get the user the app is being
            # run as. forking a process as a different user can cause children
            # to have a different owner
            with open(user_token_file, 'w') as vtoken:
                vtoken.write(client.lookup_token()['data']['id'])
        except KeyError:
            pass
    return client
