from .core import ServicectlBase, PROD_API_URL
from .schema import ChannelModel


class Channel():
    def __init__(self, channel={}, client=None, url=PROD_API_URL):
        self.client = client or ServicectlBase(url=url)
        self.channel = ChannelModel(**channel)

    def get(self, channel=None):
        """
        Fetch the details for a given channel
        """
        channel = channel or self.channel.channel

        self.channel = ChannelModel(
            **self.client._get(path='/servicectl/channels/%s' % channel))
        return self.channel

    def list(self):
        """
        Fetch a list of all the service queue channels
        """

        return self.client._get(path='/servicectl/channels')

    def add(self):
        """
        Add the current channel model as a new channel
        """

        self.channel = ChannelModel(
            **self.client._post(path='/servicectl/channels/new', data=self.channel.model_dump(exclude_unset=True)))
        return self.channel

    def save(self):
        """
        Save the current channel model
        """

        if self.channel.id_:
            # if we have an id, update
            return self.client._post(path=f'/servicectl/channels/update/{self.channel.id_}', data=self.channel.model_dump(exclude_unset=True))
        else:
            # if we do not have an id, add new
            self.add()

    def delete(self):
        """
        Delete the current channel model
        """

        return self.client._delete(path=f'/servicectl/channels/{self.channel.id_}')
