from typing import Union, Optional, List
from datetime import datetime, date, timedelta
from enum import Enum
from pydantic import BaseModel, Field
import pytz


class JobModel(BaseModel):
    class resultenum(Enum):
        failure = 'failure'
        success = 'success'
        scheduled = 'scheduled'
        died = 'died'
        canceled = 'canceled'
        killed = 'killed'

    class Config:
        use_enum_values = True

    service_queue_id: Optional[int] = None
    service: Optional[str] = None
    home: Optional[str] = None
    command: Optional[str] = None
    options: Optional[str] = None
    scheduled: Optional[Union[datetime, date]
                        ] = datetime.now(pytz.timezone('America/Los_Angeles'))
    dispatched: Optional[Union[datetime, date]] = None
    executed: Optional[Union[datetime, date]] = None
    ended: Optional[Union[datetime, date]] = None
    finished: Optional[Union[datetime, date]] = None
    kill_after: Optional[Union[datetime, date]] = None
    kill_interval: Optional[str] = None
    proc: Optional[str] = None
    run_by: Optional[int] = None
    channel: Optional[str] = 'default'
    priority: Optional[int] = 0
    instance: Optional[int] = None
    hinstance: Optional[int] = None
    after: Optional[int] = None
    parent: Optional[int] = None
    asap: Optional[int] = 0
    mem_used: Optional[int] = 0
    assigned_to_service: Optional[str] = None
    assigned_to_home: Optional[str] = None
    executed_by_service: Optional[str] = None
    executed_by_home: Optional[str] = None
    executed_by_user: Optional[str] = None
    result: Optional[resultenum] = 'scheduled'
    attempt: Optional[int] = 0
    location_hint: Optional[str] = None
    retries: Optional[int] = 0


class HistoryModel(BaseModel):
    class resultenum(Enum):
        failure = 'failure'
        success = 'success'
        scheduled = 'scheduled'
        died = 'died'
        canceled = 'canceled'
        killed = 'killed'

    class Config:
        use_enum_values = True

    home: str
    service: str
    command: str
    interval: Optional[str] = None
    result: Optional[resultenum] = None


class SearchJobModel(JobModel):
    start_scheduled: Optional[Union[datetime, date]] = None
    end_scheduled: Optional[Union[datetime, date]] = None
    start_dispatched: Optional[Union[datetime, date]] = None
    end_dispatched: Optional[Union[datetime, date]] = None
    start_executed: Optional[Union[datetime, date]] = None
    end_executed: Optional[Union[datetime, date]] = None
    start_ended: Optional[Union[datetime, date]] = None
    end_ended: Optional[Union[datetime, date]] = None
    start_finished: Optional[Union[datetime, date]] = None
    end_finished: Optional[Union[datetime, date]] = None
    start_kill_after: Optional[Union[datetime, date]] = None
    end_kill_after: Optional[Union[datetime, date]] = None


class ServiceModel(BaseModel):
    class statusenum(Enum):
        active = 'active'
        inactive = 'inactive'

    class Config:
        population_by_name = True
        use_enum_values = True

    service: Optional[str] = None
    home: Optional[str] = None
    status: Optional[statusenum] = None
    type_: Optional[str] = Field(default=None, alias="type")
    parent_service: Optional[str] = None
    parent_home: Optional[str] = None
    controller_service: Optional[str] = None
    controller_home: Optional[str] = None
    params: Optional[List] = []


class ChannelModel(BaseModel):
    class realtimeenum(Enum):
        true = 'true'
        false = 'false'

    class Config:
        population_by_name = True
        use_enum_values = True

    id_: Optional[int] = Field(default=None, alias='id')
    channel: Optional[str] = None
    max_jobs: Optional[int] = None
    retries: Optional[int] = None
    realtime: Optional[realtimeenum] = None
