from .core import ServicectlBase, PROD_API_URL
from .schema import ServiceModel


class Service():
    def __init__(self, service={}, client=None, url=PROD_API_URL):
        self.service = ServiceModel(**service)
        self.client = client or ServicectlBase(url=url)

    def sheep(self):
        """
        Fetch the cron-like services
        """

        return self.client._get(path='/servicectl/services/scheduled_sheep')

    def controller_children(self, controller_home, controller_service, stype=None):
        """
        Fetch the services that are under the control of a given controller_home:controller_service. Optionally, filter by service type
        """

        params = {'controller_home': controller_home,
                  'controller_service': controller_service}
        if stype:
            params['stype'] = stype

        return self.client._get(path='/servicectl/services/controller_children', params=params)

    def params(self):
        """
        Return service parameters
        """
        return self.service.params

    def get(self, home=None, service=None, params=True):
        """
        Fetch information about this service
        """
        home = home or self.service.home
        service = service or self.service.service

        self.service = ServiceModel(
            **self.client._get(path=f'/servicectl/services/details/{home}/{service}', params={'params': params}))
        return self.service

    def all_crons(self):
        """
        Fetch a list of cron-like services
        """
        return self.client._get(path=f'/servicectl/services/all_crons')

    def list(self, service=None, home=None, status=None, type_=None, parent_home=None, parent_service=None):
        """
        Fetch a list of cron-like services
        """
        list_filter = ServiceModel(**{
            'service': service or self.service.service,
            'home': home or self.service.home,
            'status': status or self.service.status,
            'type_': type_ or self.service.type_,
            'parent_home': parent_home or self.service.parent_home,
            'parent_service': parent_service or self.service.parent_service})
        return self.client._get(path=f'/servicectl/services', params=list_filter.model_dump(exclude_unset=True))
